package io.tswf.groovy.bettergroovy.io.tree


import groovy.transform.CompileStatic
import io.tswf.groovy.bettergroovy.exception.FileTreeCreationException
import io.tswf.groovy.bettergroovy.exception.FileTreeProviderException

@CompileStatic
class FileTrees {

    @Lazy
    static List<IFileTreeProvider> fileTreeProviders = ServiceLoader.load(IFileTreeProvider)
        .toList()

    static IFileTree getFileTree(Object source) {
        try {
            for (def fileTreeProvider : fileTreeProviders) {
                try {
                    if (fileTreeProvider.acceptsRawSource(source)) {
                        def result = fileTreeProvider.getFileTree(source)
                        if (result != null) {
                            return result
                        }
                    }
                } catch (Throwable e) {
                    throw new FileTreeProviderException(e, source, fileTreeProvider)
                }
            }
        } catch (Throwable e) {
            if (e instanceof FileTreeCreationException.Bypass) {
                throw e
            }

            throw new FileTreeCreationException(e, source)
        }

        return null
    }

}
