package io.tswf.groovy.bettergroovy.io.tree.provider

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.groovy.bettergroovy.exception.InvalidFileTreeSourceException
import io.tswf.groovy.bettergroovy.io.tree.IFileTree
import io.tswf.groovy.bettergroovy.io.tree.IFileTreeEntry
import io.tswf.groovy.bettergroovy.io.tree.IFileTreeProvider

import java.util.stream.Stream
import java.util.zip.ZipEntry
import java.util.zip.ZipFile

@CompileStatic
@AutoService(IFileTreeProvider)
class ZipFileTreeProvider implements IFileTreeProvider {

    @Override
    IFileTree getFileTree(Object source) {
        if (!acceptsRawSource(source)) {
            throw new InvalidFileTreeSourceException(this, source)
        }

        return new ZipFileTree((ZipFile) source)
    }

    @Override
    boolean acceptsRawSource(Object source) {
        return source instanceof ZipFile
    }
}

@CompileStatic
class ZipFileTree implements IFileTree {

    ZipFile zipFile

    ZipFileTree(ZipFile zipFile) {
        this.zipFile = zipFile
    }

    @Override
    IFileTreeEntry getEntry(String name) {
        return zipFile.getEntry(name)?.with { new ZipFileTreeEntry(zipFile, it) }
    }

    @Override
    Stream<IFileTreeEntry> getEntries() {
        return getEntries("")
    }

    @Override
    Stream<IFileTreeEntry> getEntries(String path) {
        def slashedPath = path.isNullOrBlank() ? "" : path.endsWith("/") ? path : "${path}/".toString()

        return zipFile.stream()
            .filter { it.name.startsWith(slashedPath) }
            .map {(IFileTreeEntry) new ZipFileTreeEntry(zipFile, it)  }
    }

    void close() throws IOException {
        zipFile.close()
    }
}

@CompileStatic
class ZipFileTreeEntry implements IFileTreeEntry {

    ZipFile zipFile
    ZipEntry zipEntry

    ZipFileTreeEntry(ZipFile zipFile, ZipEntry zipEntry) {
        this.zipFile = zipFile
        this.zipEntry = zipEntry
    }

    @Override
    String getName() {
        return zipEntry.name
    }

    @Override
    InputStream getInputStream() {
        return zipFile.getInputStream(zipEntry)
    }
}
