package io.tswf.groovy.bettergroovy.scan

import com.google.auto.service.AutoService
import groovy.transform.CompileStatic
import io.tswf.groovy.bettergroovy.transforms.validation.checkedmapconstructor.CheckedMapConstructor

import java.util.function.Supplier

@AutoService(IResourcesProvider)
@CheckedMapConstructor
@CompileStatic
class DirectoryResourceProvider implements IResourcesProvider{

    @Override
    void getResource(ResourceGetContext context) {
        def directory = context.source as File
        for (def location : context.locations) {
            location = location.stripAll()
            directory.subFile(location)?.with {
                if (exists() && isFile()) {
                    context.foundResource.add(getResourceFromFile(directory, it))
                }
            }
        }
    }

    @Override
    void getResources(ResourceScanContext context) {
        def directory = context.source as File
        def subdir = directory.subFile(context.packageName.replace(".", "/"))

        if (!subdir.exists())
            return

        context.foundResources += subdir.walkSubFiles()
            .map {getResourceFromFile(directory, it)}
            .filter {context.filter.test(it)}
    }

    @Override
    boolean acceptsContext(ResourceScanContext context) {
        return acceptsSource(context.source)
    }

    @Override
    boolean acceptsContext(ResourceGetContext context) {
        return acceptsSource(context.source)
    }

    boolean acceptsSource(Object source) {
        return source instanceof File && source.isDirectory()
    }

    Resource getResourceFromFile(File directory, File file) {
        return new Resource(
                name: file.name,
                fullPath: file.getRelativeFrom(directory).path,
                owner: directory,
                inputStreamSupplier: {file.getInputStream()} as Supplier<InputStream>
        )
    }
}
